#include <msp/time/utils.h>
#include "demo.h"

using namespace std;
using namespace Msp;

Demo::Demo(Graphics::Window &window, Graphics::GLContext &gl_ctx, DataFile::Collection &r):
	resources(r),
	view(window, gl_ctx),
	anim_action(anim_player),
	music_source(0),
	streamer(0),
	music_io(0),
	music_decoder(0)
{
	things["window"] = static_cast<GL::View *>(&view);

	sequencer.add_static_action(anim_action);
	sequencer.signal_finished.connect(signal_finished);
}

Demo::~Demo()
{
	delete streamer;
	delete music_decoder;
	delete music_io;
	delete music_source;
}

void Demo::set_fixed_framerate(float fps)
{
	frame_interval = Time::sec/fps;
}

void Demo::enable_music()
{
	if(!music_source)
		music_source = new AL::Source;
	if(!streamer)
		streamer = new AL::Streamer(*music_source);
}

void Demo::play_music(const string &fn)
{
	if(!streamer)
		return;

	streamer->stop();
	delete music_decoder;
	delete music_io;
	music_io = resources.open_raw(fn);
	music_decoder = AL::SoundDecoder::open_io(*music_io);
	float beat = sequencer.get_current_beat();
	if(beat)
	{
		float time = beat*60/sequencer.get_beats_per_minute();
		unsigned bytes_per_sec = music_decoder->get_frequency()*get_unit_size(music_decoder->get_format());
		music_decoder->seek(time*bytes_per_sec);
	}
	streamer->play(*music_decoder);
}

void Demo::tick()
{
	Time::TimeStamp t = Time::now();
	Time::TimeDelta dt = (last_tick ? frame_interval ? frame_interval : t-last_tick : Time::zero);
	last_tick = t;

	if(frame_interval)
		next_frame = t+frame_interval;

	sequencer.tick(dt);
	if(streamer)
		streamer->tick();
	view.render();
}

void Demo::seek(const Time::TimeDelta &pos)
{
	sequencer.seek(pos*sequencer.get_beats_per_minute()/Time::min);
}


Demo::AnimationAction::AnimationAction(GL::AnimationPlayer &p):
	player(p)
{ }

void Demo::AnimationAction::tick(float, float d)
{
	player.tick(d*Time::sec);
}
